//=========================================================================//
//-------------------------------------------------------------------------//
// $Id: ds1621.h,v 1.3 2001/12/12 05:43:23 lanconel Exp $                                                                    //
//-------------------------------------------------------------------------//
//  Copyright (C) 1998-2001  Claudio Lanconelli                            //
//                                                                         //
//  e-mail: lancos@libero.it                                               //
//  http://www.LancOS.com                                                  //
//-------------------------------------------------------------------------//
//                                                                         //
// This program is free software; you can redistribute it and/or           //
// modify it under the terms of the GNU  General Public License            //
// as published by the Free Software Foundation; either version2 of        //
// the License, or (at your option) any later version.                     //
//                                                                         //
// This program is distributed in the hope that it will be useful,         //
// but WITHOUT ANY WARRANTY; without even the implied warranty of          //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       //
// General Public License for more details.                                //
//                                                                         //
// You should have received a copy of the GNU  General Public License      //
// along with this program (see COPYING);     if not, write to the         //
// Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. //
//                                                                         //
//-------------------------------------------------------------------------//
//=========================================================================//

#ifndef	_DS1621_H
#define	_DS1621_H

#include "types.h"
#include "i2cbus.h"

#define	BASE_ADDRESS	0x90        /* Temperature I2CBus address */

/** Configuration register bits **/
#define DONE_BIT    7       /* Conversion done bit (0 = conversion in progress) */
#define THF_BIT     6       /* Temperature high flag */
#define TLF_BIT     5       /* Temperature low flag */
#define NVB_BIT     4       /* EEPROM busy flag (1 = write in progress) */
#define	RES_BIT		3		/* reserved bit */
#define POL_BIT     1       /* Output polarity (1 = active high) */
#define ONESHOT_BIT 0       /* One shot mode (0 = free running) */

class DS1621
{
 public:		//------------------------------- public
	DS1621();
	virtual ~DS1621();

	void Init(bool one_shot = true);
	void Open(int port = 1);
	void Close();

	int StartConversion();
	int StopConversion();
	int ReadConfiguration(UBYTE &val);
	int WriteConfiguration(UBYTE val);
	int ReadHighThreshold(short &val);
	int WriteHighThreshold(short val);
	int ReadLowThreshold(short &val);
	int WriteLowThreshold(short val);
	int ReadTemperature(short &val);
	int ReadCounter(UBYTE &val);
	int ReadSlope(UBYTE &val);
	int WaitConversion();

	int GetAddress() const
	{
		return (i2c_address - BASE_ADDRESS) / 2 + 1;
	}

	void SetAddress(int addr)
	{
		if (addr >= 1 && addr <= 8)
			i2c_address = BASE_ADDRESS + (addr - 1) * 2;
	}

	int GetError() const
	{
		return error_code;
	}

 protected:		//------------------------------- protected

	signed short read_sword_register(UBYTE cmd_code);
	UWORD read_word_register(UBYTE cmd_code);
	signed char read_sbyte_register(UBYTE cmd_code);
	UBYTE read_byte_register(UBYTE cmd_code);

	void write_sword_register(UBYTE cmd_code, signed short val);
	void write_word_register(UBYTE cmd_code, UWORD val);
	void write_sbyte_register(UBYTE cmd_code, signed char val);
	void write_byte_register(UBYTE cmd_code, UBYTE val);

	void write_command(UBYTE cmd_code);

	int const CmdStartConversion;
	int const CmdStopConversion;
	int const CmdAccessConfig;
	int const CmdAccessTH;
	int const CmdAccessTL;
	int const CmdReadTemp;
	int const CmdReadCounter;
	int const CmdReadSlope;

 private:		//------------------------------- private

	void i2c_read_packet(unsigned int len);
	void i2c_write_packet(unsigned int len);

	int	error_code;

	int	i2c_address;
	UBYTE	i2c_buffer[256];

	I2CBus	i2c;
};

#endif
